#include "stdafx.h"
#include <stdio.h>
#include <algorithm>
#include "winioctl.h"
#include "USB_ioctl.h"
#include "USB_defs.h"
#include "LUSB.h"


TLUSB_BASE::TLUSB_BASE(unsigned int ioctl_retries)
    : MaxIoctlRetries(ioctl_retries), LastIoctlRetries(0),
      hDevice(INVALID_HANDLE_VALUE), LastErrorNumber(LUSB_OK)
    {
    memset(&ControlOverlapped, 0, sizeof(OVERLAPPED));
    memset(&ReadBulkOverlapped, 0, sizeof(OVERLAPPED));
    memset(&WriteBulkOverlapped, 0, sizeof(OVERLAPPED));
    ControlOverlapped.hEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
    ReadBulkOverlapped.hEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
    WriteBulkOverlapped.hEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
    }

TLUSB_BASE::~TLUSB_BASE()
    {
    CancelIo();
    CloseLDevice();
    if (ControlOverlapped.hEvent)   CloseHandle(ControlOverlapped.hEvent);
    if (ReadBulkOverlapped.hEvent)  CloseHandle(ReadBulkOverlapped.hEvent);
    if (WriteBulkOverlapped.hEvent) CloseHandle(WriteBulkOverlapped.hEvent);
    }

bool __fastcall TLUSB_BASE::OpenLDevice(BYTE VirtualSlot)
    {
    char DeviceName[18];
    memset(&DeviceName, 0, sizeof(DeviceName));
    //    USB
    snprintf(DeviceName, sizeof(DeviceName), "\\\\.\\LDev%d", VirtualSlot);
    return OpenLDeviceByName(DeviceName);
    }

bool __fastcall TLUSB_BASE::OpenLDeviceByName(LPTSTR DeviceName)
    {
    if (!DeviceName || !strlen(DeviceName))
        return LastErrorNumber = LUSB_ERROR_INVALID_PARAMETERS, false;

    if (!CloseLDevice()) return false;

    LastErrorNumber = LUSB_OK;
    hDevice = CreateFile(DeviceName, GENERIC_READ | GENERIC_WRITE, 0,
            NULL, OPEN_EXISTING, FILE_FLAG_OVERLAPPED, NULL);
    if (INVALID_HANDLE_VALUE == hDevice)
        LastErrorNumber = LUSB_ERROR_OPEN_DEVICE;

    return (LUSB_OK == LastErrorNumber);
    }

bool __fastcall TLUSB_BASE::CloseLDevice()
    {
    if (hDevice != INVALID_HANDLE_VALUE)
        {
        if (!CloseHandle(hDevice))
            return LastErrorNumber = LUSB_ERROR_CLOSE_DEVICE, false;
        hDevice = INVALID_HANDLE_VALUE;
        }
    LastErrorNumber = LUSB_OK;
    return true;
    }

bool __fastcall TLUSB_BASE::CancelIo()
    {
    if (INVALID_HANDLE_VALUE == hDevice)
        return LastErrorNumber = LUSB_ERROR_DEVICE_CLOSED, false;

    LastErrorNumber = LUSB_OK;
    if (!::CancelIo(hDevice))
        LastErrorNumber = LUSB_ERROR_CANCEL_IO;

    return (LUSB_OK == LastErrorNumber);
    }

bool __fastcall TLUSB_BASE::ControlLDevice(DWORD ControlCode, _ControlRequest * CurrentRequest, BYTE DataBuffer[], DWORD DataBufferLength, DWORD Timeout)
    {
    TCSLock lock(cs_control);
    if (INVALID_HANDLE_VALUE == hDevice)
        return LastErrorNumber = LUSB_ERROR_DEVICE_CLOSED, false;

    ControlCode = DIOC_SEND_COMMAND; // 
    LastIoctlRetries = 0;
    LastErrorNumber = LUSB_ERROR_TIMEOUT_EXPIRED;

    while (LastIoctlRetries < MaxIoctlRetries)
        {
        LastErrorNumber = LUSB_OK;
        DWORD BytesReturned = 0;
        if (!DeviceIoControl(hDevice, ControlCode, CurrentRequest, sizeof(_ControlRequest),
                DataBuffer, DataBufferLength, &BytesReturned, &ControlOverlapped))
            {
            if (ERROR_IO_PENDING == GetLastError())
                { /* I/O Pending */
                if (WaitForSingleObject(ControlOverlapped.hEvent, Timeout) != WAIT_OBJECT_0)
                    {
                    ::CancelIo(hDevice);
                    LastErrorNumber = LUSB_ERROR_TIMEOUT_EXPIRED;
                    }

                if (!GetOverlappedResult(hDevice, &ControlOverlapped, &BytesReturned, TRUE))
                    {
                    if (LUSB_OK == LastErrorNumber)
                        LastErrorNumber = LUSB_ERROR_ASYNC_ERROR;
                    }
                }
            else
                { /* Real error */
                LastErrorNumber = LUSB_ERROR_ASYNC_DATA_TRNSFER;
                }
            } /* if !DeviceIoControl */

        if ((LUSB_OK == LastErrorNumber) && (BytesReturned != DataBufferLength))
            LastErrorNumber = LUSB_ERROR_ASYNC_DATA_TRNSFER_LENGTH;

        if (LUSB_OK == LastErrorNumber)
            break;

        LastIoctlRetries++;

        } /* while retries */

    return (LUSB_OK == LastErrorNumber);
    }

bool __fastcall TLUSB_BASE::CommandControl(_ControlRequest *CurrentRequest)
    {
    BYTE DataBuffer[1];
    return ControlLDevice(DIOC_SEND_COMMAND, CurrentRequest, DataBuffer, 0, 1000);
    }

bool __fastcall TLUSB_BASE::ReadBulk(BYTE DataBuffer[], DWORD DataBufferLength, DWORD* BytesRead, DWORD Timeout)
    {
    if (!DataBuffer || !BytesRead)
        return LastErrorNumber = LUSB_ERROR_INVALID_PARAMETERS, false;
    TCSLock lock(cs_readbulk);
    if (INVALID_HANDLE_VALUE == hDevice)
        return LastErrorNumber = LUSB_ERROR_DEVICE_CLOSED, false;

    LastErrorNumber = LUSB_OK;
    *BytesRead = 0;

    if (!ReadFile(hDevice, DataBuffer, DataBufferLength, BytesRead, &ReadBulkOverlapped))
        {
        if (ERROR_IO_PENDING == GetLastError())
            {
            if (WaitForSingleObject(ReadBulkOverlapped.hEvent, Timeout) != WAIT_OBJECT_0)
                {
                ::CancelIo(hDevice);
                LastErrorNumber = LUSB_ERROR_TIMEOUT_EXPIRED;
                }

            if (!GetOverlappedResult(hDevice, &ReadBulkOverlapped, BytesRead, TRUE))
                {
                if (LUSB_OK == LastErrorNumber)
                    LastErrorNumber = LUSB_ERROR_ASYNC_ERROR;
                }
            }
        else
            {
            LastErrorNumber = LUSB_ERROR_ASYNC_DATA_TRNSFER;
            }
        }

#if 0
    if ((LUSB_OK == LastErrorNumber) && (*BytesRead != DataBufferLength))
        LastErrorNumber = LUSB_ERROR_ASYNC_DATA_TRNSFER_LENGTH;
#endif

    return (LUSB_OK == LastErrorNumber);
    }

bool __fastcall TLUSB_BASE::WriteBulk(BYTE DataBuffer[], DWORD DataBufferLength, DWORD* BytesWritten, DWORD Timeout)
    {
    if (!DataBuffer || !BytesWritten)
        return LastErrorNumber = LUSB_ERROR_INVALID_PARAMETERS, false;
    TCSLock lock(cs_writebulk);
    if (INVALID_HANDLE_VALUE == hDevice)
        return LastErrorNumber = LUSB_ERROR_DEVICE_CLOSED, false;

    LastErrorNumber = LUSB_OK;
    *BytesWritten = 0;

    if (!WriteFile(hDevice, DataBuffer, DataBufferLength, BytesWritten, &WriteBulkOverlapped))
        {
        if (ERROR_IO_PENDING == GetLastError())
            {
            if (WaitForSingleObject(WriteBulkOverlapped.hEvent, Timeout) != WAIT_OBJECT_0)
                {
                ::CancelIo(hDevice);
                LastErrorNumber = LUSB_ERROR_TIMEOUT_EXPIRED;
                }

            if (!GetOverlappedResult(hDevice, &WriteBulkOverlapped, BytesWritten, TRUE))
                {
                if (LUSB_OK == LastErrorNumber)
                    LastErrorNumber = LUSB_ERROR_ASYNC_ERROR;
                }
            }
        else
            {
            LastErrorNumber = LUSB_ERROR_ASYNC_DATA_TRNSFER;
            }
        }

#if 0
    if ((LUSB_OK == LastErrorNumber) && (*BytesWritten != DataBufferLength))
        LastErrorNumber = LUSB_ERROR_ASYNC_DATA_TRNSFER_LENGTH;
#endif

    return (LUSB_OK == LastErrorNumber);
    }

bool __fastcall TLUSB_BASE::GetModuleName(char mname[], DWORD Length)
    {
    if (!mname)
        return LastErrorNumber = LUSB_ERROR_INVALID_PARAMETERS, false;

    if (INVALID_HANDLE_VALUE == hDevice)
        return LastErrorNumber = LUSB_ERROR_DEVICE_CLOSED, false;

    _ControlRequest Request;
    Request.RequestDirectionIN = true;
    Request.bRequest = 11;
    Request.wValue = 0;
    Request.wIndex = 0;
    memset(mname, 0, Length);
    return ControlLDevice(0, &Request, (BYTE *)mname, Length, 1000);
    }
