/*================================================================================================*
 *   LTR-
 *================================================================================================*/

#include <stdio.h>
#include <stdint.h>
#include <windows.h>
#include <winsock2.h>
#include <getopt.h>
#include "ltr\\include\\ltrapidefine.h"
#include "ltr\\include\\ltrapi.h"

/*================================================================================================*/

/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
int die(const char* fmt, ...)
    {
    va_list va;
    va_start(va, fmt);
    vfprintf(stderr, fmt, va);
    fputs("\n", stderr);
    va_end(va);
    exit(1);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void help(void)
    {
    puts(
        "Usage: ltrserver_ctl [-s server_host] [-p server_port] {shutdown|restart}\n"
        "    -s server_host     Host name or IP address of ltrserver (default: localhost)\n"
        "    -p server_port     TCP port the server is listening on  (default: 11111)\n"
        "    shutdown           Shutdown ltrserver\n"
        "    restart            Restart ltrserver\n"
        );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int main(int argc, char* argv[])
    {
    WSADATA wsaData;
    int option;
    char* eol;
    struct in_addr server_ip;
    uint16_t server_port = 11111;
    enum { DO_NOTHING = 0, DO_RESTART, DO_SHUTDOWN } action = DO_NOTHING;
    TLTR ltr;

    server_ip.s_addr = htonl(INADDR_LOOPBACK);

    if (WSAStartup(0x202, &wsaData))
        die("Cannot initialize winsock");

    while (0 <= (option = getopt(argc, argv, "h?s:p:")))
        {
        switch (option)
            {
            case 's':
                {
                struct hostent* host = gethostbyname(optarg);
                if (!host || (host->h_addrtype != AF_INET) || (!host->h_addr_list[0]))
                    die("-%c: cannot get IP address of %s", option, optarg);
                server_ip.s_addr = *(u_long*)host->h_addr_list[0];
                }
                break;
            case 'p':
                {
                unsigned long uval = (uint16_t)strtoul(optarg, &eol, 10);
                if (*eol || !uval || (uval > UINT16_MAX))
                    die("-%c: invalid port %s", option, optarg);
                server_port = (uint16_t)uval;
                }
                break;
            case 'h':
            case '?':
                help();
                return 0;
            }
        }

    if (argc < 2)
        {
        help();
        return 0;
        }

    if (!stricmp(argv[1], "restart"))
        {
        action = DO_RESTART;
        }
    else if (!stricmp(argv[1], "shutdown"))
        {
        action = DO_SHUTDOWN;
        }
    else
        {
        help();
        return 0;
        }

    /* Connect */
    if (LTR_OK != LTR_Init(&ltr))
        die("Cannot initialize LTRAPI");
    printf("Connecting to ltrserver at %s:%u...", inet_ntoa(server_ip), server_port);
    ltr.saddr = ntohl(server_ip.s_addr);
    ltr.sport = server_port;
    strncpy(ltr.csn, CSN_SERVER_CONTROL, sizeof(ltr.csn));
    ltr.cc = CC_CONTROL;
    if (LTR_OK == LTR_OpenEx(&ltr, 5000))
        {
        puts("ok");
        }
    else
        {
        die("cannot connect");
        }

    /* Execute action */
    switch (action)
        {
        case DO_RESTART:
            printf("Restarting ltrserver... ");
            puts((LTR_OK == LTR_ServerRestart(&ltr)) ? "ok" : "command failed");
            break;
        case DO_SHUTDOWN:
            printf("Shutting down ltrserver... ");
            if (LTR_OK == LTR_ServerShutdown(&ltr))
                {
                DWORD dummy, count = 10;
                LTR_SetTimeout(&ltr, 500);
                while ((LTR_OK == LTR_GetServerVersion(&ltr, &dummy)) && count--)
                    {
                    Sleep(250);
                    }
                puts((count) ? "ok" : "wait timed out");
                Sleep(250);
                }
            else
                {
                puts("command failed");
                }
            break;
        default:
            break;
        }

    LTR_Close(&ltr);
    return 0;
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
