/*================================================================================================*
 *     ltrserver.
 *  INI-      thread-safe  
 *  .
 *================================================================================================*/

#ifndef LtrConfigH
#define LtrConfigH

#include <vector>
#include "LtrIniFile.h"

/*================================================================================================*/
#define CFG_NO_SAVE false
#define CFG_SAVE    true
/*================================================================================================*/

/*================================================================================================*/
template <typename T>
class TLtrLockedVar
    {
private:
    TCritical cs;
    T value;
public:
    T get()
        {
        TCSLock lock(cs);
        return value;
        }
    T xchg(T new_value)
        {
        TCSLock lock(cs);
        T old_value = value;
        value = new_value;
        return old_value;
        }
    TLtrLockedVar& operator= (const T& rhs)
        {
        TCSLock lock(cs);
        value = rhs;
        return *this;
        }
    };
/*================================================================================================*/

/*================================================================================================*/
class TLtrConfig
    {
public:
    typedef struct
        {
        DWORD ip;
        DWORD flags;
        }
        IPCrateRec;
    typedef std::vector<IPCrateRec> IPCrateVector;
    typedef struct
        {
        RECT rect;
        int active_tab;
        bool maximized;
        bool valid;
        }
        VisualState;
private:
    TCritical cs_visual;
    TCritical cs_ip;
    TLtrIniFile ini;
    int max_log_level;

    TLtrLockedVar<DWORD> server_priority;
    TLtrLockedVar<bool> minimize_to_tray;
    TLtrLockedVar<bool> start_minimized;
    TLtrLockedVar<bool> auto_connect_dynamic;
    TLtrLockedVar<bool> enable_auto_discovery;
    TLtrLockedVar<int> log_level;
    TLtrLockedVar<AnsiString> language;
    TLtrLockedVar<AnsiString> default_ttf;
    TLtrLockedVar<AnsiString> log_file_name;
    TLtrLockedVar<DWORD> listen_addr;
    TLtrLockedVar<WORD> listen_port;
    TLtrLockedVar<bool> multiconnect_to_module;
    TLtrLockedVar<bool> enable_long_tstamp;
    TLtrLockedVar<DWORD> crate_connect_timeout, crate_command_timeout, crate_ping_timeout,
        crate_idle_timeout, crate_reconnect_delay;
    TLtrLockedVar<int> crate_connect_attempts, crate_ping_attempts;
public:
    //  
    void Open();
    void Close();
    void SetMaxLogLevel(int maxlevel)
        { max_log_level = maxlevel; }

    //    
    DWORD ServerPriority()      { return server_priority.get(); }
    bool MinimizeToTray()       { return minimize_to_tray.get(); }
    bool StartMinimized()       { return start_minimized.get(); }
    bool AutoConnectDynamic()   { return auto_connect_dynamic.get(); }
    bool EnableAutoDiscovery()  { return enable_auto_discovery.get(); }
    int LogLevel()              { return log_level.get(); }
    AnsiString Language()       { return language.get(); }
    DWORD ListenAddr()          { return listen_addr.get(); }
    WORD ListenPort()           { return listen_port.get(); }
    AnsiString DefaultTTF()     { return default_ttf.get(); }
    AnsiString LogFileName()    { return log_file_name.get(); }
    bool Multiconnect()         { return multiconnect_to_module.get(); }
    bool LongTStampEnabled()    { return enable_long_tstamp.get(); }
    DWORD CrateConnectTimeout() { return crate_connect_timeout.get(); }
    DWORD CrateCommandTimeout() { return crate_command_timeout.get(); }
    DWORD CratePingTimeout()    { return crate_ping_timeout.get(); }
    DWORD CrateIdleTimeout()    { return crate_idle_timeout.get(); }
    DWORD CrateReconnectDelay() { return crate_reconnect_delay.get(); }
    int CrateConnectAttempts()  { return crate_connect_attempts.get(); }
    int CratePingAttempts()     { return crate_ping_attempts.get(); }

    //    
    //    ( InterlockedExchange)
    //  bSave ,       .
    DWORD ServerPriority(DWORD dwNewVal, bool bSave);
    bool MinimizeToTray(bool bNewVal, bool bSave);
    bool StartMinimized(bool bNewVal, bool bSave);
    bool AutoConnectDynamic(bool bNewVal, bool bSave);
    bool EnableAutoDiscovery(bool bNewVal, bool bSave);
    int LogLevel(int iNewVal, bool bSave);
    void Language(const AnsiString& sNewVal, bool bSave);
    DWORD ListenAddr(DWORD dwNewVal, bool bSave);
    WORD ListenPort(WORD wNewVal, bool bSave);
    bool Multiconnect(bool bNewVal, bool bSave);
    bool LongTStampEnabled(bool bNewVal, bool bSave);
    DWORD CrateConnectTimeout(DWORD dwNewVal, bool bSave);
    DWORD CrateCommandTimeout(DWORD dwNewVal, bool bSave);
    DWORD CratePingTimeout(DWORD dwNewVal, bool bSave);
    DWORD CrateIdleTimeout(DWORD dwNewVal, bool bSave);
    DWORD CrateReconnectDelay(DWORD dwNewVal, bool bSave);
    int CrateConnectAttempts(int iNewVal, bool bSave);
    int CratePingAttempts(int iNewVal, bool bSave);

    //  - ,     
    void LoadVisualState(VisualState& state);
    void SaveVisualState(const VisualState& state);
    void LoadIPCrates(TUniqueAddrMultiList& list);
    bool AddIPCrate(TUniqueAddrMultiList& list, DWORD ip, DWORD flags, bool bSave);
    bool ChangeIPCrate(TUniqueAddrMultiList& list, DWORD old_ip, DWORD new_ip,
        DWORD new_flags, bool bSave);
    bool SetIPCrateFlags(TUniqueAddrMultiList& list, DWORD ip, DWORD mask, DWORD bits, bool bSave);
    bool DeleteIPCrate(TUniqueAddrMultiList& list, DWORD ip, bool bSave);
    AnsiString GetTTFFileName(const AnsiString& section);

    };
/*================================================================================================*/

#endif
