#ifndef LtrCrateH
#define LtrCrateH

#include "common.h"
#include "LtrCrateModule.h" 

/*================================================================================================*/
class TCrate
    {
protected:
    volatile bool Suspended;
    HANDLE hStopEvent;
    HANDLE ThreadTerminationInUse;              // mutex
    volatile t_CrateState CurState;
    MessageQueue<MsgControl>& refMQ_Control;
    MessageQueue<MsgDisplay>& refMQ_Display;
    bool special_code;

    virtual bool StartServiceThreads(void) = 0;
    virtual void StopServiceThreads(void) = 0;

public:
    BYTE CrateType;
    WORD protocol_version;                      // (TCP)  
    DWORD ip_addr;                              // (TCP) IP-
    DWORD usb_devinst;                          // (USB)  DevInst
    char path[1024];                            // (USB)    
                                                // (TCP) IP-   
    char csn[17];                               //   
    char name[256];
    volatile int sflags;                        //   
    t_USB_ConnInfo usb_info;                    //    USB
    BYTE hw_revision;                           //   
    WORD baud_rate;                             //   
    //                                          //
    DWORD second_mark_counter;                  //     LTR43
    DWORD start_mark_counter;                   //     LTR43
    int unixtime_seq;
    __int64 unixtime_val;
    //                                          //
    SYSTEMTIME start_time;                      //
    volatile __int64 total_count_recv;          //      
    volatile __int64 total_count_send;          //      

    bool ReadRawData;                           //       17

    TCrateModule Module[MODULE_ARRAYLEN];
    HANDLE hClientPutEvent;

    TCrate(MessageQueue<MsgControl>& mqControl, MessageQueue<MsgDisplay>& mqDisplay);
    virtual ~TCrate(void);

    /* Callback-      
           .     . */
    bool ParseDataFromCrate(LPDWORD lpBuffer, DWORD NumDWords);
    DWORD PrepareDataForCrate(LPDWORD lpBuffer, DWORD MaxDWords, DWORD &SkipCount);

//    bool IsStopped(void)
//        { return (WSAWaitForMultipleEvents(1, &hStopEvent, FALSE, 0, FALSE) == WSA_WAIT_EVENT_0); }

    /*        */
    t_CrateState GetState(void)
        { return (this != NULL) ? CurState : CRATE_ST_NULL; }
    virtual void SetState(t_CrateState new_state, bool NotifyThreads = true) = 0;

    /*    */
    virtual BYTE GetConnType(void) = 0;
    virtual bool Startup(BYTE * FPGAData, bool use_default_ttf) = 0;
    virtual void SuspendCrate(void) = 0;
    virtual void ResumeCrate(void) = 0;

    /*      */
    virtual void CancelIO(void) = 0;
    virtual bool CallApplication(DWORD address) = 0;
    virtual bool GetArray(DWORD mode, DWORD address, BYTE *data, DWORD size) = 0;
    virtual bool PutArray(DWORD mode, DWORD address, BYTE *data, DWORD size) = 0;
    virtual bool GetModuleDescription(DWORD mode, TDESCRIPTION_LTR010 *description) = 0;
    virtual bool SetModuleDescription(DWORD mode, TDESCRIPTION_LTR010 *description) = 0;
    virtual int PingAndGetFailCount(DWORD timeout) = 0;

    /*   (  ) */
    virtual void RestartIdleTimer(void)     {}
    virtual void StopIdleTimer(void)        {}
    virtual HANDLE GetIdleTimerHandle(void) { return NULL; }

protected:
    typedef enum
        {
        module_init_normal,
        module_init_no_reset
        }
        module_init_mode_t;

    void LockAllModules()
        {
        for (int i = 0; i < MODULE_ARRAYLEN; i++)
            {
            Module[i].pvt_Lock();
            }
        }

    void UnlockAllModules()
        {
        for (int i = 0; i < MODULE_ARRAYLEN; i++)
            {
            Module[i].pvt_CrateUpdate();
            Module[i].pvt_Unlock();
            }
        }

    bool ProcessCrateError(DWORD Data);
    void __fastcall ProcessSyncMark(WORD mark_val);

    void InitLockedModule(int ModuleSlot, WORD new_mid, module_init_mode_t init_mode);
    };

//---------------------------------------------------------------------------
class TCrateUSB : public TCrate
    {
private:
    TThread *recv_buf_thread;
    TThread *send_buf_thread;
    HANDLE DeviceHANDLE;
    TLTRUSB *ltr_USB;

    bool InitCrate(BYTE * FPGAData, bool use_default_ttf);
    AnsiString IoRetryStr(void);
    virtual bool StartServiceThreads(void);
    virtual void StopServiceThreads(void);
public:
    TCrateUSB(MessageQueue<MsgControl>& mqControl, MessageQueue<MsgDisplay>& mqDisplay);
    virtual ~TCrateUSB(void);
    virtual void SetState(t_CrateState new_state, bool NotifyThreads = true);
    virtual BYTE GetConnType(void);
    virtual bool Startup(BYTE * FPGAData, bool use_default_ttf);
    virtual void SuspendCrate(void);
    virtual void ResumeCrate(void);

    virtual void CancelIO(void);
    virtual bool CallApplication(DWORD address);
    virtual bool GetArray(DWORD mode, DWORD address, BYTE *data, DWORD size);
    virtual bool PutArray(DWORD mode, DWORD address, BYTE *data, DWORD size);
    virtual bool GetModuleDescription(DWORD mode, TDESCRIPTION_LTR010 *description);
    virtual bool SetModuleDescription(DWORD mode, TDESCRIPTION_LTR010 *description);
    virtual int PingAndGetFailCount(DWORD timeout);

friend
    class TReadUSBCrateThread;
friend
    class TWriteUSBCrateThread;
    };

//---------------------------------------------------------------------------
class TCrateIP : public TCrate
    {
private:
    int ping_fail_count;
    TThread *data_thread;
    TLTRIP *ltr_ip;

    bool InitCrate(void);
    virtual bool StartServiceThreads(void);
    virtual void StopServiceThreads(void);
public:
    TCrateIP(MessageQueue<MsgControl>& mqControl, MessageQueue<MsgDisplay>& mqDisplay,
        DWORD IdleTimeout);
    virtual ~TCrateIP(void);
    virtual void SetState(t_CrateState new_state, bool NotifyThreads = true);
    virtual BYTE GetConnType(void);
    virtual bool Startup(BYTE * FPGAData, bool use_default_ttf);
    virtual void SuspendCrate(void);
    virtual void ResumeCrate(void);

    virtual void CancelIO(void);
    virtual bool CallApplication(DWORD address);
    virtual bool GetArray(DWORD mode, DWORD address, BYTE *data, DWORD size);
    virtual bool PutArray(DWORD mode, DWORD address, BYTE *data, DWORD size);
    virtual bool GetModuleDescription(DWORD mode, TDESCRIPTION_LTR010 *description);
    virtual bool SetModuleDescription(DWORD mode, TDESCRIPTION_LTR010 *description);
    virtual int PingAndGetFailCount(DWORD timeout);

    virtual AnsiString GetErrorMessage(void);

    /*   */
    virtual void RestartIdleTimer(void)     { ltr_ip->RestartIdleTimer(); }
    virtual void StopIdleTimer(void)        { ltr_ip->StopIdleTimer(); }
    virtual HANDLE GetIdleTimerHandle(void) { return ltr_ip->GetIdleTimerHandle(); }

friend
    class TIPCrateDataThread;
    };
/*================================================================================================*/

#endif
