/*================================================================================================*
 *      LTR     
 *================================================================================================*/

#include <vcl.h>
#include <stdio.h>
#pragma hdrstop

#include "LtrCrateModule.h"
#include "LtrClient.h"

/*================================================================================================*/
const TCrateModule::bufrec_t TCrateModule::recv_zero =
    {
    /* buf */           NULL,
    /* buf_len */       0,
    /* last_offset */   0,
    /* last_count */    0
    };

const TCrateModule::ringrec_t TCrateModule::send_zero =
    {
    /* buf */           NULL,
    /* buf_len */       0,
    /* read_ptr */      NULL,
    /* write_ptr */     NULL,
    /* end_ptr */       NULL
    };
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
TCrateModule::TCrateModule()
    : present(false), need_init(false), mid(0),
      fifo_len(0), count_send(0), count_recv(0),
      raise_CrateGetEvent(false), raise_CratePutEvent(false)
    {
    recv = recv_zero;
    send = send_zero;
#if !PER_CLIENT_CRATE_EVENTS
    hCrateGetEvent = CreateEvent(NULL, EVT_AUTO_RESET, EVT_NONSIGNALED, NULL);
    hCratePutEvent = CreateEvent(NULL, EVT_AUTO_RESET, EVT_NONSIGNALED, NULL);
#endif
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TCrateModule::~TCrateModule()
    {
    Delete();
#if !PER_CLIENT_CRATE_EVENTS
    CloseHandle(hCratePutEvent);
    CloseHandle(hCrateGetEvent);
#endif
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TCrateModule::err_t TCrateModule::Create(TCrateModule::access_t access)
    {
    SCOPED_LOCK;
    if (present) return eAlreadyExists;
    err_t result = eOk;

    mid = 0xFFFF;

    if (send.buf) delete[] send.buf;
    if (recv.buf) delete[] recv.buf;
    recv = recv_zero;
    send = send_zero;

    recv.buf = new DWORD[RecvBufLen];
    if (recv.buf)
        {
        recv.buf_len = RecvBufLen;
        }
    else
        {
        result = eInternal;
        }

    if (access != accReadOnly)
        {
        send.buf = new DWORD[SendBufLen];
        if (!send.buf)
            result = eInternal;
        }

    if (send.buf)
        {
        send.buf_len = SendBufLen;
        send.read_ptr = send.write_ptr = send.buf;
        send.end_ptr = send.buf + send.buf_len;
        }

    count_recv = count_send = 0;
    raise_CrateGetEvent = raise_CratePutEvent = false;
    fifo_len = 0;

    if (eOk == result)
        {
        present = true;
        need_init = (access != accReadOnly);
        }
    else
        {
        Delete();
        }
    return result;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TCrateModule::Delete()
    {
    SCOPED_LOCK;
    present = false;
    need_init = false;
    mid = 0;
    if (send.buf) delete[] send.buf;
    if (recv.buf) delete[] recv.buf;
    recv = recv_zero;
    send = send_zero;
    /*   ,        */
#if !PER_CLIENT_CRATE_EVENTS
    SetEvent(hCrateGetEvent);
    SetEvent(hCratePutEvent);
#else
    for (client_set::iterator pcl = connected_clients.begin(); pcl != connected_clients.end();
        pcl++)
        {
        (*pcl)->SetModuleGetEvent();
        (*pcl)->SetModulePutEvent();
        }
#endif
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TCrateModule::SendBufEmpty()
    {
    SCOPED_LOCK;
    return (!present || (send.write_ptr == send.read_ptr));
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TCrateModule::err_t TCrateModule::GetMID(WORD& out_mid)
    {
    SCOPED_LOCK;
    if (present)
        {
        out_mid = mid;
        return eOk;
        }
    else
        {
        out_mid = 0;
        return eNotPresent;
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TCrateModule::err_t TCrateModule::GetStatistics(TCrateModule::stat_t& stat)
    {
    SCOPED_LOCK;
    if (!present) return eNotPresent;
    stat.mid = mid;
    stat.count_send = count_send;
    stat.count_recv = count_recv;
    stat.count_FIFO_recv = recv.last_count;
    stat.fifo_len = fifo_len;
    return eOk;
    }
/*------------------------------------------------------------------------------------------------*/

#if !PER_CLIENT_CRATE_EVENTS
/*------------------------------------------------------------------------------------------------*/
TCrateModule::err_t TCrateModule::ClientGetInit(TCrateModule::bufrec_t& client_rec)
    {
    SCOPED_LOCK;
    if (present)
        {
        client_rec.last_offset = recv.last_offset;
        client_rec.last_count = recv.last_count;
        return eOk;
        }
    else
        {
        client_rec.last_offset = 0;
        client_rec.last_count = 0;
        return eNotPresent;
        }
    }
/*------------------------------------------------------------------------------------------------*/
#endif

/*------------------------------------------------------------------------------------------------*/
TCrateModule::err_t TCrateModule::ClientGetLen
    (
    TCrateModule::bufrec_t& client_rec,
    DWORD& out_len
    )
    {
    out_len = 0;
    SCOPED_LOCK;
    if (!present) return eNotPresent;
    __int64 unread = recv.last_count - client_rec.last_count;
    out_len = (unread > client_rec.buf_len) ? client_rec.buf_len : unread;
    if (unread > recv.buf_len)
        return eOverflow;
    return eOk;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TCrateModule::err_t TCrateModule::ClientGet
    (
    TCrateModule::bufrec_t& client_rec,
    DWORD& out_len
    )
    {
    out_len = 0;
    if (!client_rec.buf) return eInternal;
    SCOPED_LOCK;
    if (!present) return eNotPresent;
    __int64 unread = recv.last_count - client_rec.last_count;
    DWORD read_len = (unread > client_rec.buf_len) ? client_rec.buf_len : unread;
    if (unread > recv.buf_len)
        {
        // ,       
        client_rec.last_offset = recv.last_offset;
        client_rec.last_count = recv.last_count;
        return eOverflow;
        }
    else if (unread > 0)
        {
        //  
        DWORD len1 = std::min(recv.buf_len - client_rec.last_offset, read_len);
        DWORD len2 = read_len - len1;
        if (len1 > 0)
            {
            memcpy(&client_rec.buf[0], &recv.buf[client_rec.last_offset], len1 * sizeof(DWORD));
            client_rec.last_offset += len1;
            }
        if (len2 > 0)
            {
            memcpy(&client_rec.buf[len1], &recv.buf[0], len2 * sizeof(DWORD));
            client_rec.last_offset = len2;
            }
        if (client_rec.last_offset >= recv.buf_len)
            client_rec.last_offset -= recv.buf_len;

        client_rec.last_count += read_len;
        out_len = read_len;
        }
    return eOk;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TCrateModule::err_t TCrateModule::ClientPut
    (
    const DWORD* src_buf,
    DWORD count,
    DWORD& out_len,
    DWORD and_mask,
    DWORD or_mask
    )
    {
    out_len = 0;
    if (count == 0) return eOk;
    if (!src_buf) return eInternal;
    SCOPED_LOCK;
    if (!present) return eNotPresent;
    if (!send.buf) return eWrongOp;
    ptrdiff_t avail = send.read_ptr - send.write_ptr - 1;
    if (avail < 0)
        avail += send.buf_len;
    if (avail > 0)
        {
        //    
        DWORD to_copy = out_len = std::min(count, (DWORD)avail);
        while (to_copy--)
            {
            *send.write_ptr++ = (*src_buf++ & and_mask) | or_mask;
            if (send.write_ptr >= send.end_ptr)
                send.write_ptr = send.buf;
            }
        }
    return eOk;
    }
/*------------------------------------------------------------------------------------------------*/

#if PER_CLIENT_CRATE_EVENTS
/*------------------------------------------------------------------------------------------------*/
TCrateModule::err_t TCrateModule::ConnectClient
    (TClient* client)
    {
    SCOPED_LOCK;
    if (present)
        {
        connected_clients.insert(client);
        client->read_rec.last_offset = recv.last_offset;
        client->read_rec.last_count = recv.last_count;
        return eOk;
        }
    else
        {
        client->read_rec.last_offset = 0;
        client->read_rec.last_count = 0;
        return eNotPresent;
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TCrateModule::DisconnectClient
    (TClient* client)
    {
    SCOPED_LOCK;
    connected_clients.erase(client);
    }
/*------------------------------------------------------------------------------------------------*/
#endif

/*------------------------------------------------------------------------------------------------*/
//       CrateGet/CratePut
void TCrateModule::pvt_CrateUpdate()
    {
    if (!present) return;
#if !PER_CLIENT_CRATE_EVENTS
    if (raise_CratePutEvent)
        {
        raise_CratePutEvent = false;
        SetEvent(hCratePutEvent);
        }
    if (raise_CrateGetEvent)
        {
        raise_CrateGetEvent = false;
        SetEvent(hCrateGetEvent);
        }
#else
    if (raise_CratePutEvent || raise_CrateGetEvent)
        {
        for (client_set::iterator pcl = connected_clients.begin(); pcl != connected_clients.end();
            pcl++)
            {
            if (raise_CrateGetEvent) (*pcl)->SetModuleGetEvent();
            if (raise_CratePutEvent) (*pcl)->SetModulePutEvent();
            }
        raise_CratePutEvent = raise_CrateGetEvent = false;
        }
#endif
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
