/*================================================================================================*
 *      LTR     
 *================================================================================================*/

#ifndef LtrCrateModuleH
#define LtrCrateModuleH

#include "common.h"
#if PER_CLIENT_CRATE_EVENTS
#include <set>
#endif

/*================================================================================================*/
#define LTR34_BUFFER_ALERT_SIZE             2000000 //was 1572864
#define MODULE_USE_CRITICAL                 1

#define LTR_MID(mid)                        (((mid) & 0xFF) | (((mid) & 0xFF) << 8))
/*================================================================================================*/

#if MODULE_USE_CRITICAL
#include "critical.h"
#define SCOPED_LOCK TCSLock lock(cs)
#else
#define SCOPED_LOCK
#endif

/*================================================================================================*/
class TCrateModule
    {
public:
    static const DWORD RecvBufLen = 1048576;
    static const DWORD SendBufLen = 131072;

    //    ( )   
    typedef enum err_enum
        {
        eOk = 0,                // 
        eNotPresent = -1,       //   ()
        eOverflow = -2,         //      (ClientGetLen, ClientGet)
        eAlreadyExists = -3,    //    (Create)
        eWrongOp = -4,          //   (ClientPut  read-only )
        eInternal = -5,         //   (   ..)
        eGeneral = -100         //  
        }
        err_t;

    //    
    typedef enum access_enum
        {
        accNormal = 0,          //  
        accReadOnly = 1         //     ( => eWrongOp)
        }
        access_t;
    
    //       */
    typedef struct
        {
        WORD mid;
        __int64 count_send;
        __int64 count_recv;
        __int64 count_FIFO_recv;
        DWORD fifo_len;
        }
        stat_t;

    //     " "
    typedef struct
        {
        DWORD* buf;
        DWORD buf_len;
        DWORD last_offset;
        __int64 last_count;
        }
        bufrec_t;

    //     
    typedef struct
        {
        DWORD* buf;
        DWORD buf_len;
        DWORD* read_ptr;
        DWORD* write_ptr;
        DWORD* end_ptr;
        }
        ringrec_t;

    static const bufrec_t recv_zero;
    static const ringrec_t send_zero;

private:
#if MODULE_USE_CRITICAL
    TCritical cs;
#endif    
    bool present;                           //   
    bool need_init;                         // ,   
    WORD mid;                               //  
    bufrec_t recv;
    ringrec_t send;
    //    LTR34
    __int64 count_recv;                     //      
    __int64 count_send;                     //   ,   
    DWORD fifo_len;                         //   FIFO  
    //     
    bool raise_CrateGetEvent;
    bool raise_CratePutEvent;
public:
    TCrateModule();
    ~TCrateModule();
    err_t Create(TCrateModule::access_t access);
    void Delete();
    bool Present()      { return present; }
    bool SendBufEmpty();
    err_t GetMID(WORD& out_mid);
    err_t GetStatistics(stat_t& stat);
    err_t ClientGetLen(bufrec_t& client_rec, DWORD& out_len);
    err_t ClientGet(bufrec_t& client_rec, DWORD& out_len);
    err_t ClientPut(const DWORD* src_buf, DWORD count, DWORD& out_len,
        DWORD and_mask = 0xFFFFFFFF, DWORD or_mask = 0);

#if !PER_CLIENT_CRATE_EVENTS
    // ,    
    HANDLE hCrateGetEvent;
    HANDLE hCratePutEvent;
    err_t ClientGetInit(bufrec_t& client_rec);
#else
    err_t ConnectClient(TClient* client);
    void DisconnectClient(TClient* client);
#endif

private: // for friends only

#if PER_CLIENT_CRATE_EVENTS
    typedef std::set<TClient*> client_set;
    client_set connected_clients;
#endif

#if MODULE_USE_CRITICAL
    void pvt_Lock()         { cs.Enter(); }
    void pvt_Unlock()       { cs.Leave(); }
#endif

/*------------------------------------------------------------------------------------------------*/
void pvt_InitMID(WORD new_mid)
    {
    if (!present) return;
    mid = new_mid;
    count_recv = count_send = 0;
    fifo_len = 0;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool pvt_GetAndClearInitFlag()
    {
    if (!present) return false;
    if (!need_init) return false;
    need_init = false;
    //       
    count_recv = count_send = 0;
    send.read_ptr = send.write_ptr; //   
    return true;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
//       
bool pvt_CrateDataReady()
    {
    return (
        present
        && (send.write_ptr != send.read_ptr) && send.buf
        && ((mid != LTR_MID(34)) || (count_send - count_recv < LTR34_BUFFER_ALERT_SIZE))
        );
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
//        .
//         CrateDataReady()
// (     0)
DWORD pvt_CrateGet()
    {
    if (!present) return 0;
    if ((send.write_ptr == send.read_ptr) || !send.buf)
        return 0;

    DWORD data_word = *send.read_ptr;
    count_send++;
    //     LTR34
    if ((mid == LTR_MID(34)) && ((data_word & 0xC000) == LTR010CMD))
        { //  data_word - ,        
          //       (   FIFO)
        if ((data_word & 0xC0C0) == LTR010CMD_RESET)
            {
            count_send = count_recv = 0;
            }
        else
            {
            count_send--;
            }
        }

    //   
    if (++send.read_ptr >= send.end_ptr)
        send.read_ptr = send.buf;
    raise_CrateGetEvent = true;
    return data_word;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
//        
void pvt_CratePut(DWORD data_word)
    {
    if (!present) return;
    recv.buf[recv.last_offset] = data_word;
    if (++recv.last_offset >= recv.buf_len)
        recv.last_offset = 0;
    //     
    recv.last_count++;
    count_recv++;
    raise_CratePutEvent = true;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
//       CrateGet/CratePut
void pvt_CrateUpdate();
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void pvt_CrateUpdateLTR34
    (DWORD count, DWORD fifo)
    {
    if (!present) return;
    count_recv += count;
    fifo_len = fifo;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void pvt_CrateUpdateLTR34
    (DWORD fifo_state)
    {
    if (!present) return;
    __int64 delta = (count_send - count_recv) >> 10;
    if (delta > 0xFFFF)
        {
        fifo_len = 0xFFFF;
        }
    else if (delta < 0)
        {
        fifo_len = 0;
        }
    else
        {
        fifo_len = delta & 0xFFFF;
        }
    fifo_len |= fifo_state;
    }
/*------------------------------------------------------------------------------------------------*/

friend class TCrate;
    };
/*================================================================================================*/
#endif
