#ifndef MessageQueueH
#define MessageQueueH

#include <queue>
#include <windows.h>
#include "critical.h"

/*================================================================================================*/
/*  T    void uniq(),     
   (  AnsiString,        ) */

template <class T>
class MessageQueue
    {
private:
    TCritical cs;
    std::queue<T> q;
    HANDLE hPostEvent;
public:
    void __fastcall put(const T& msg)
        {
        TCSLock lock(cs);
        T tmp = msg;
        tmp.uniq();
        q.push(tmp);
        SetEvent(hPostEvent);
        }

    bool __fastcall get(T* msg, DWORD wait_time = 0)
        {
        bool wait = (wait_time > 0);
        if (wait)
            {
            TCSLock lock(cs); // lock for access to q.empty()
            wait = q.empty();
            }

        // wait in unlocked state
        if (wait)
            WaitForSingleObject(hPostEvent, wait_time);

        TCSLock lock(cs); // lock for access to q
        if (q.empty())
            return false;
        *msg = q.front();
        q.pop();
        msg->uniq();
        return true;
        }

    MessageQueue()
        {
        hPostEvent = CreateEvent(NULL, FALSE /*auto*/, FALSE /*nonsignaled*/, NULL);
        }

    ~MessageQueue()
        {
        CloseHandle(hPostEvent);
        }
    };
/*================================================================================================*/
#endif
