#include "addrlist.h"

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
TAddrListNode::TAddrListNode(void)
    {
    dwIP = INADDR_NONE;
    iStatus = IP_STATUS_INVALID;
    szSerial[0] = '\0';
    dwFlags = 0;
    next = NULL;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TAddrListNode::TAddrListNode(const TAddrListNode &Source)
    {
    dwIP = Source.dwIP;
    iStatus = Source.iStatus;
    memcpy(szSerial, Source.szSerial, sizeof(szSerial));
    dwFlags = Source.dwFlags;
    next = NULL;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TAddrListNode& TAddrListNode::operator=(const TAddrListNode &Source)
    {
    if (this != &Source)
        {
        dwIP = Source.dwIP;
        iStatus = Source.iStatus;
        memcpy(szSerial, Source.szSerial, sizeof(szSerial));
        dwFlags = Source.dwFlags;
        /* next is unchanged (only data is copied) */
        }
    return *this;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TAddrListNode::SetAddr(DWORD dwSetIP)
    {
    if ((dwSetIP == INADDR_NONE) || (dwSetIP == INADDR_ANY))
        return false;
    dwIP = dwSetIP;
    return true;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TAddrListNode::SetStatus(int iNewStatus, LPCSTR lpszSerial)
    {
    iStatus = iNewStatus;
    if (lpszSerial)
        {
        strncpy(szSerial, lpszSerial, sizeof(szSerial) - 1);
        szSerial[sizeof(szSerial) - 1] = '\0';
        }
    else
        {
        szSerial[0] = '\0';
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TAddrList::TAddrList(void)
    {
    first = last = NULL;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TAddrList::TAddrList(const TAddrList &Source)
    {
    MkCopy(Source, first, last);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TAddrList& TAddrList::operator=(const TAddrList &Source)
    {
    if (this != &Source)
        {
        TAddrListNode *new_first, *new_last;
        MkCopy(Source, new_first, new_last);
        Clear();
        first = new_first;
        last = new_last;
        }
    return *this;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TAddrList::~TAddrList(void)
    {
    Clear();
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TAddrList::MkCopy(const TAddrList &Source, TAddrListNode *&NewFirst, TAddrListNode *&NewLast)
    {
    NewFirst = NewLast = NULL;
    TAddrListNode *node = Source.first;
    while (node)
        {
        TAddrListNode *newnode = new TAddrListNode(*node);
        if (!newnode)
            break;
        if (!NewFirst)
            {
            NewFirst = newnode;
            }
        else
            {
            NewLast->next = newnode;
            }
        NewLast = newnode;
        node = node->next;
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TAddrListNode* TAddrList::GetFirstNode(void) const
    {
    return first;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TAddrListNode* TAddrList::FindFirst(DWORD dwFindIP) const
    {
    if ((dwFindIP == INADDR_NONE) || (dwFindIP == INADDR_ANY))
        return NULL;
    TAddrListNode *node = first;
    while (node)
        {
        if (node->dwIP == dwFindIP)
            return node;
        node = node->next;
        }
    return NULL;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TAddrList::Add(DWORD dwAddIP, DWORD dwFlags)
    {
    if ((dwAddIP == INADDR_NONE) || (dwAddIP == INADDR_ANY))
        return false;
    TAddrListNode *newnode = new TAddrListNode;
    if (!newnode) return false;
    if (!newnode->SetAddr(dwAddIP))
        {
        delete newnode;
        return false;
        }
    newnode->SetStatus(IP_STATUS_OFFLINE, NULL);
    newnode->dwFlags = dwFlags;
    if (!first || !last)
        {
        first = newnode;
        }
    else
        {
        last->next = newnode;
        }
    last = newnode;
    return true;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TAddrList::Delete(DWORD dwDeleteIP)
    {
    bool result = false;
    TAddrListNode *node = first;
    TAddrListNode *prevnode = NULL;
    while (node)
        {
        TAddrListNode *nextnode = node->next;
        if (node->dwIP == dwDeleteIP)
            {
            /* A node that is in use cannot be deleted */
            if ((node->iStatus == IP_STATUS_ONLINE) || (node->iStatus == IP_STATUS_CONNECTING))
                return false;
            if (prevnode)
                prevnode->next = nextnode;
            if (node == first)
                first = nextnode;
            if (node == last)
                last = prevnode;
            delete node;
            result = true;
            }
        else /* node->dwIP != deDeleteIP */
            {
            prevnode = node;
            }
        node = nextnode;
        }
    return result;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TAddrList::Clear(void)
    {
    TAddrListNode *second;
    while (first)
        {
        second = first->next;
        delete first;
        first = second;
        }
    last = NULL;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TUniqueAddrMultiList::TUniqueAddrMultiList(void)
    {
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TUniqueAddrMultiList::~TUniqueAddrMultiList(void)
    {
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int TUniqueAddrMultiList::Find(DWORD dwFindIP, TAddrListNode *pNodeCopy)
    {
    TCSLock lock(cs);
    int found_index = -1;
    for (int k = 0; k < ADDRLIST_MULTI_NUM; k++)
        {
        TAddrListNode *node = List[k].FindFirst(dwFindIP);
        if (node)
            {
            found_index = k;
            if (pNodeCopy)
                *pNodeCopy = *node;
            break;
            }
        }
    return found_index;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TUniqueAddrMultiList::AddUnique(int ListIndex, DWORD dwAddIP, DWORD dwFlags)
    {
    if ((ListIndex < 0) || (ListIndex >= ADDRLIST_MULTI_NUM))
        return false;
    bool result = true;
    TCSLock lock(cs);
    for (int k = 0; k < ADDRLIST_MULTI_NUM; k++)
        {
        if (List[k].FindFirst(dwAddIP))
            {
            result = false;
            break;
            }
        }
    if (result)
        result = List[ListIndex].Add(dwAddIP, dwFlags);
    return result;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TUniqueAddrMultiList::ReplaceUnique(int ListIndex, DWORD dwOldIP, DWORD dwNewIP, DWORD dwFlags)
    {
    if ((ListIndex < 0) || (ListIndex >= ADDRLIST_MULTI_NUM))
        return false;
    TCSLock lock(cs);
    /* Find dwOldIP */
    TAddrListNode *node = List[ListIndex].FindFirst(dwOldIP);
    if (!node) return false;
    /* A node that is in use cannot be replaced */
    if ((node->iStatus == IP_STATUS_ONLINE) || (node->iStatus == IP_STATUS_CONNECTING))
        return false;
    if (node->dwIP != dwNewIP)
        {
        /* Look if dwNewIP does not yet exist in any list */
        for (int k = 0; k < ADDRLIST_MULTI_NUM; k++)
            {
            if (List[k].FindFirst(dwNewIP))
                return false;
            }
        node->dwIP = dwNewIP;
        }
    node->dwFlags = dwFlags;
    return true;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TUniqueAddrMultiList::Delete(int ListIndex, DWORD dwDeleteIP)
    {
    if ((ListIndex < 0) || (ListIndex >= ADDRLIST_MULTI_NUM))
        return false;
    TCSLock lock(cs);
    bool result = List[ListIndex].Delete(dwDeleteIP);
    return result;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
bool TUniqueAddrMultiList::SetStatus(DWORD dwFindIP, int iNewStatus, LPCSTR lpszSerial)
    {
    bool result = false;
    TCSLock lock(cs);
    for (int k = 0; k < ADDRLIST_MULTI_NUM; k++)
        {
        /* Since the IPs in the list are unique, we only have to look for the first occurence */
        TAddrListNode *node = List[k].FindFirst(dwFindIP);
        if (node)
            {
            node->SetStatus(iNewStatus, lpszSerial);
            result = true;
            break;
            }
        }
    return result;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int TUniqueAddrMultiList::SetFlags(DWORD dwFindIP, DWORD dwMask, DWORD dwNewBits, DWORD* pdwNewFlags)
    {
    int found_index = -1;
    TCSLock lock(cs);
    for (int k = 0; k < ADDRLIST_MULTI_NUM; k++)
        {
        /* Since the IPs in the list are unique, we only have to look for the first occurence */
        TAddrListNode *node = List[k].FindFirst(dwFindIP);
        if (node)
            {
            node->dwFlags = (node->dwFlags & ~dwMask) | (dwNewBits & dwMask);
            if (pdwNewFlags)
                *pdwNewFlags = node->dwFlags;
            found_index = k;
            break;
            }
        }
    return found_index;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TUniqueAddrMultiList::ClearList(int ListIndex)
    {
    if ((ListIndex < 0) || (ListIndex >= ADDRLIST_MULTI_NUM))
        return;
    TCSLock lock(cs);
    List[ListIndex].Clear();
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
TAddrList* TUniqueAddrMultiList::ExportList(int ListIndex)
    {
    if ((ListIndex < 0) || (ListIndex >= ADDRLIST_MULTI_NUM))
        return NULL;
    TCSLock lock(cs);
    TAddrList *copy = new TAddrList(List[ListIndex]);
    return copy;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TUniqueAddrMultiList::Clear(void)
    {
    TCSLock lock(cs);
    for (int k = 0; k < ADDRLIST_MULTI_NUM; k++)
        {
        List[k].Clear();
        }
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
