#include "debugdump.h"
#if DEBUGDUMP_ENABLE

/*================================================================================================*/
#define DWORDS_PER_LINE     8
#define MAX_SHOW_DWORDS     32

TDebugDump g_DebugDump;
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
void TDebugDump::Dump
    (LPCVOID Data, DWORD DataSize, LPCSTR Comment)
    {
    if (!Data || !Comment) return;

    struct { DWORD idx, len; } block[2];
    int blocks;
    block[0].idx = 0;

    DWORD num_dwords = DataSize / sizeof(DWORD);
    if (num_dwords <= MAX_SHOW_DWORDS)
        {
        blocks = 1;
        block[0].len = num_dwords;
        }
    else
        {
        blocks = 2;
        block[0].len = MAX_SHOW_DWORDS / 2;
        block[1].len = MAX_SHOW_DWORDS - block[0].len;
        block[1].idx = num_dwords - block[1].len;
        }

    TCSLock lock(cs);
    if (!f)
        {
        f = fopen("debugdump.txt", "w");
        if (!f) return;
        }

    SYSTEMTIME st;
    GetLocalTime(&st);
    fprintf(f, "%02d.%02d.%02d %02d:%02d:%02d.%03d : %lu byte(s) %s\n",
        st.wDay, st.wMonth, st.wYear % 100, st.wHour, st.wMinute, st.wSecond,
        st.wMilliseconds, DataSize, Comment);
    bool open_line = false;
    for (int i = 0; i < blocks; i++)
        {
        if (i) fprintf(f, "...\n");
        for (int j = 0; j < block[i].len; j++)
            {
            fprintf(f, "%08lX ", ((const DWORD*)Data)[block[i].idx + j]);
            open_line = (((j + 1) % DWORDS_PER_LINE) != 0);
            if (!open_line)
                fprintf(f, "\n");
            }
        }
    if (open_line)
        fprintf(f, "\n");
    if (num_dwords * sizeof(DWORD) < DataSize)
        {
        fprintf(f, "+");
        for (DWORD i = num_dwords * sizeof(DWORD); i < DataSize; i++)
            fprintf(f, " %02X", ((const BYTE*)Data)[i]);
        fprintf(f, "\n");
        }
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
#endif
