#include <stdio.h>

/* Read .rc file, get all string IDs and make an .h file with a #define for each
 * Syntax: mkids file.rc file.h first_id
 */

enum states
    {
    st_none,
    st_head1,  /* after STRINGTABLE */
    st_active, /* after BEGIN */
    };

struct strlist
    {
    struct strlist *next;
    char txt[1];
    };

void die(const char *msg)
    {
    fprintf(stderr, "MKIDS: %s\n", msg);
    exit(1);
    }

trim(char *str)
    {
    int k;

    if (!str) return;

    k = strlen(str) - 1;
    while ((k >= 0) && ((str[k] == '\t') || (str[k] == ' ') || (str[k] == '\r') || (str[k] == '\n')))
        {
        str[k--] = '\0';
        }
    k = 0;
    while ((str[k] == '\t') || (str[k] == ' '))
        {
        k++;
        }
    strcpy(str, str + k);
    }

main(int argc, char *argv[])
    {
    unsigned int count = 0;
    FILE *fin, *fout;
    char line[1024];
    int state = st_none;
    struct strlist *idstart = NULL, *idend = NULL, *id;

    if (argc != 4)
        die("Invalid number of arguments");
    sscanf(argv[3], "%u", &count);
    if (!count)
        die("Invalid initial resource ID");
    fin = fopen(argv[1], "r");
    if (!fin)
        die("Cannot open input file");
    while (!feof(fin))
        {
        fgets(line, sizeof(line), fin);
        trim(line);
        switch (state)
            {
            case st_none:
                if (strstr(line, "STRINGTABLE") == line)
                    state = st_head1;
                break;
            case st_head1:
                if (strstr(line, "BEGIN") == line)
                    state = st_active;
                break;
            case st_active:
                if (strstr(line, "END") == line)
                    {
                    state = st_none;
                    }
                else
                    {
                    int k = 0;
                    while (isalnum(line[k]) || (line[k] == '_'))
                        {
                        k++;
                        }
                    line[k] = '\0';
                    if (!line[0])
                        break;
                    if (!idstart || !idend)
                        {
                        idstart = (struct strlist*)malloc(sizeof(struct strlist) + strlen(line));
                        idstart->next = NULL;
                        strcpy(idstart->txt, line);
                        idend = idstart;
                        }
                    else
                        {
                        id = idstart;
                        int unique = 1;
                        while (unique && id)
                            {
                            if (!strcmp(line, id->txt))
                                unique = 0;
                            id = id->next;
                            }
                        if (unique)
                            {
                            id = (struct strlist*)malloc(sizeof(struct strlist) + strlen(line));
                            id->next = NULL;
                            strcpy(id->txt, line);
                            idend->next = id;
                            idend = id;
                            }
                        }
                    }
                break;    
            }
        }
    fclose(fin);

    fout = fopen(argv[2], "w");
    if (!fout)
        die("Cannot open output file");
    fputs("#ifndef StringResH\n", fout);
    fputs("#define StringResH\n", fout);
    fprintf(fout, "\n/* Autogenerated from %s, do not edit */\n\n", argv[1]);
    id = idstart;
    while (id)
        {
        fprintf(fout, "#define %s %u\n", id->txt, count++);
        id = id->next;
        }
    fputs("\n#endif\n", fout);
    fclose(fout);
    }
